package luokat;
import java.io.*;
import java.util.Random;

import fi.jyu.mit.ohj2.Mjonot;

/** Pelaaja-luokka. Pitää kirjaa pelaajien id:stä, 
 * @author Sakari
 * @version 1.3.2025
 *
 */
public class Pelaaja {
    
    
    private String nimi;
    private int id;
    private static int seuraavaId = 1;
    
    
    

//    /** Pelaaja-luokan muodostaja
//     * @param nimi Pelaajan nimi
//     */
//    public Pelaaja(String nimi) {
//        this.nimi = nimi;
//    }
//    
    
    /** Metodi, joka hakee pelaajan nimen
     * @return Pelaajan nimen
     * @example
     * <pre name="test">
     *   Pelaaja pasi = new Pelaaja();
     *   pasi.vastaaPasi();
     *   pasi.getNimi() =R= "pasi .*";
     * </pre>
     */
    public String getNimi() {
        return this.nimi;
    }
    
    
    /** Metodi, joka hakee pelaajan id:n
     * @return pelaajan id:n
     */
    public int getId() {
        return this.id;
    }
    
    /**
     * Asettaa tunnusnumeron 
     * @param nro miksi numeroksi asetetaan
     */
    public void setId(int nro) {
        this.id = nro;
        if (this.id >= seuraavaId) seuraavaId = this.id +1;
    }
    
    /** Antaa seuraavalle pelaajalle oikean id:n
     * @return uuden pelaajan id:n
     * @example
     * <pre name="test">
     * Pelaaja makipeto = new Pelaaja();
     * Pelaaja makimahoton = new Pelaaja();
     * Pelaaja vyonsolki = new Pelaaja();
     * makipeto.getId() === 0;
     * makipeto.rekisteroi();
     * makimahoton.rekisteroi();
     * vyonsolki.rekisteroi();
     * makipeto.getId() === 1;
     * makimahoton.getId() === 2;
     * vyonsolki.getId() === 3;
     * </pre>
     */
    public int rekisteroi() {
        this.id = seuraavaId;
        seuraavaId++;
        return id;
    }
    
    /**Palauttaa radan tiedot merkkijonona,
     * jonka voi tallentaa tiedostoon.
     * @return merkkijonon, joka kirjoitetaan tiedostoon.
     *  @example
     * <pre name="test">
     *   Pelaaja pelaaja = new Pelaaja();
     *   pelaaja.parse("   3  |  makipeto");
     *   pelaaja.toString() === "3|makipeto";
     * </pre>  
     */

    @Override
    public String toString() {
        return ("" + this.id + "|" + this.nimi);
    }
    
    

    /** Selvittää pelaajan tiedot riviltä, jossa 
     * erottimena "|".
     * Katsoo, että seuraavaId on suurempi kuin tuleva Id
     * @param rivi mistä otetaan pelaajan tiedot 
     * @example
     * <pre name="test">
     *   Pelaaja peluri = new Pelaaja();
     *   peluri.parse("   1  |  Makipeto");
     *   peluri.getId() === 1;
     *   peluri.toString() === "1|Makipeto";
     *
     *   peluri.rekisteroi();
     *   int n = peluri.getId();
     *   peluri.parse(""+(n+20));       // Otetaan merkkijonosta vain tunnusnumero
     *   peluri.rekisteroi();           // ja tarkistetaan että seuraavalla kertaa tulee yhtä isompi
     *   peluri.getId() === n+20+1;
     *     
     * </pre>
     */

    public void parse(String rivi) {
        var sb = new StringBuilder(rivi);
        setId(Mjonot.erota(sb, '|', getId()));
        this.nimi = Mjonot.erota(sb, '|', this.nimi);
    }
    
    @Override 
    public boolean equals(Object pelaaja) {
        if (pelaaja == null) return false;
        return this.toString().equals(pelaaja.toString());
    }
    
    
    
    @Override 
    public int hashCode() {
        return this.id;
    }
    
    
    
    
    
    
    /** Apumetodi, jolla sijoitetaan "pasi + arvottu nimi" pelaajalle.
     * @param arvottuNimi pelaajalle asetettava arvottu nimi
     */
    public void vastaaPasi(String arvottuNimi) {
        this.nimi=arvottuNimi;
    }
    
    /** 
     * Ylläolevan apumetodin apumetodi, joka arpoo nimen.
     */
    public void vastaaPasi() {
        String uusiNimi = arvoNimi();
        vastaaPasi(uusiNimi);
    }
    
    /** Apualiohjelma joka arpoo pelaajalle 
     * 6-kirjaimisen nimen.
     * @return arvottu 6 kirjaiminen nimi.
     * @example
     * <pre name="test">
     * Pelaaja pasi = new Pelaaja();
     * pasi.arvoNimi() =R= "pasi .*";
     * </pre>
     */
    public String arvoNimi() {
        Random r = new Random();
        StringBuilder uusiNimi = new StringBuilder();
        uusiNimi.append("pasi ");
        String aakkoset = "abcdefghijklmnopqrstuvxyz";
        for (int i = 0; i < 6; i++) {
            uusiNimi.append(aakkoset.charAt(r.nextInt(aakkoset.length())));
        }
        return uusiNimi.toString();
    }
    
    /**
     * Tulostetaan henkilön tiedot
     * @param out tietovirta johon tulostetaan
     */
    public void tulosta(PrintStream out) {
        out.println(String.format("%03d", this.id, 3) + "  " + this.nimi); 
    }
    
    /**
     * Tulostetaan henkilön tiedot
     * @param os tietovirta johon tulostetaan
     */
    public void tulosta(OutputStream os) {
        tulosta(new PrintStream(os));
    }
    

    
    
    /** Testipääohjelma Pelaaja-luokalle. 
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        Pelaaja pasi = new Pelaaja();
        System.out.println(pasi.getNimi());
        pasi.rekisteroi();
        System.out.println(pasi.getId());
        pasi.vastaaPasi();
        System.out.println(pasi.getNimi());
        pasi.tulosta(System.out);
    }

}
