package luokat;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.NoSuchElementException;

/** Pelaajat-luokka. Pitää huolta pelaajalistasta.
 * Osaa lisätä ja poistaa jäsenen 
 * @author Sakari
 * @version 1.3.2025
 *
 */
public class Pelaajat implements Iterable<Pelaaja> {
    
    private static final int MAX_JASENIA   = 4;
    private boolean muutettu = false;
    private int lkm =0;
    private String kokoNimi = "";
    private String tiedostonPerusNimi = "pelaajat";
    private Pelaaja [] pelaajaLista = new Pelaaja[MAX_JASENIA];
    
    

    /**
     * Parametriton muodostaja Pelaajat 
     * (Oletusmuodostaja)
     */
    public Pelaajat() {
        
    }
    
    /**
     * Palauttaa Pelaajien lukumäärän
     * @return Pelaajien lukumäärä
     */
    public int getLkm() {
        return lkm;
    }

    /**
     * Lisää uuden Pelaaja-olion tietorakenteeseen.  
     * @param uusi lisättävän jäsenen viite.  Huom tietorakenne muuttuu omistajaksi
     * @throws SailoException jos pelaajalista on jo täynnä
     * @example
     * <pre name="test">
     * #THROWS SailoException 
     * Pelaajat pelurit = new Pelaajat();
     * Pelaaja pasi1 = new Pelaaja(), pasi2 = new Pelaaja();
     * pelurit.getLkm() === 0;
     * pelurit.lisaaPelaaja(pasi1); pelurit.getLkm() === 1;
     * pelurit.lisaaPelaaja(pasi2); pelurit.getLkm() === 2;
     * pelurit.lisaaPelaaja(pasi1); pelurit.getLkm() === 3;
     * pelurit.annaPelaaja(0) === pasi1;
     * pelurit.annaPelaaja(1) === pasi2;
     * pelurit.annaPelaaja(2) === pasi1;
     * pelurit.annaPelaaja(1) == pasi1 === false;
     * pelurit.annaPelaaja(1) == pasi2 === true;
     * pelurit.annaPelaaja(3) === pasi1; #THROWS IndexOutOfBoundsException 
     * pelurit.lisaaPelaaja(pasi1); pelurit.getLkm() === 4;
     * pelurit.lisaaPelaaja(pasi1);  #THROWS SailoException
     * </pre>
     */
    public void lisaaPelaaja(Pelaaja uusi) throws SailoException{
        if (this.lkm >= pelaajaLista.length) throw new SailoException("Liikaa pelaajia!!");
        pelaajaLista[lkm] = uusi;
        lkm++;
        muutettu = true;
    }
    
    /** Aliohjelma joka poistaa pelaajan pelaajalistasta, 
     * mikäli pelaaja ylipäätään on pelaajalistassa.
     * @param poistettava pelaaja joka yritetään poistaa
     * @example
     * <pre name="test">
     * #THROWS SailoException
     * Pelaajat pelurit = new Pelaajat();
     * Pelaaja pasi1 = new Pelaaja(), pasi2 = new Pelaaja(), pasi3 = new Pelaaja();
     * pasi1.rekisteroi();
     * pasi2.rekisteroi();
     * pasi3.rekisteroi();
     * pelurit.lisaaPelaaja(pasi1);
     * pelurit.lisaaPelaaja(pasi2);
     * pelurit.lisaaPelaaja(pasi3);
     * pelurit.getLkm() === 3;
     * pelurit.poistaPelaaja(pasi2);
     * pelurit.getLkm() === 2;
     * pelurit.poistaPelaaja(pasi3);
     * pelurit.getLkm() === 1;
     * pelurit.poistaPelaaja(pasi1);
     * pelurit.getLkm() === 0;
     * </pre>
     */
    public void poistaPelaaja(Pelaaja poistettava) {
        for (int i = 0; i<pelaajaLista.length; i++) {
            if (pelaajaLista[i].getId() == poistettava.getId()) 
            {
                Pelaaja[] tilapaisLista = new Pelaaja[pelaajaLista.length];
                for (int j = 0; j<i; j++) {                         //kopioidaan kohtaa i aiemmat pelaajat tilapäislistaan.
                    tilapaisLista[j] = pelaajaLista[j];
                }
                for (int k = i; k<tilapaisLista.length-1; k++) {    //kopioidaan i:n jälkeiset pelaajat tilapäislistaan
                    if (pelaajaLista[k+1] != null) {                //tarkistetaan onko null
                        tilapaisLista[k] = pelaajaLista[k+1];       //jos ei null, kopioidaan tilapäiseen
                    }
                    else break;                                     //kun löytyy null (pelaajien välissä ei voi olla null)
                }
                pelaajaLista = tilapaisLista;                       
                lkm--;                                              
                return;
            }
        }
        return; //TODO TÄHÄN JOKIN VAROITUS; 
    }
    
    /** Palauttaa i:nnen jäsenen viitteen. 
     * @param i int-indeksi jonka kohdalla oleva Pelaaja
     * halutaan.
     * @return etsityn Pelaaja-olion.
     * @throws IndexOutOfBoundsException jos annettu indeksi i
     * on laiton. 
     */
    public Pelaaja annaPelaaja(int i) throws IndexOutOfBoundsException{
        if (i<0 || i>=lkm) throw new IndexOutOfBoundsException("Laiton indeksi: " + i);
        return pelaajaLista[i];
    }
    
    
    /**
     * @return tiedoston koko nimi
     */
    public String getKokoNimi() {
        return this.kokoNimi;
    }
    
    
    /**
     * @return "oman" tiedoston perusnimen
     */
    public String getTiedostonPerusNimi() {
        return this.tiedostonPerusNimi;
    }
    
    /**Asetetaan tiedoston nimi josta luetaan 
     * @param nimi tiedoston nimi 
     */
    public void setTiedostonPerusNimi(String nimi) {
        tiedostonPerusNimi = nimi;
    }
    
    /**
     * @return tiedoston nimen - Tänne tallennetaan!
     */
    public String getTiedostonNimi() {
        return getTiedostonPerusNimi() + ".dat";
    }

    
    /**
     * @return varakopiotiedoston nimi
     */
    public String getBakNimi() {
        return tiedostonPerusNimi + ".bak";
    }


    
    /**
     * Lukee pelaajat tiedostosta. (Kesken)
     * @param hakemisto missä pelaajat.dat sijaitsee
     * @throws SailoException jos lukeminen epäonnistuu
     *@example
     * <pre name="test">
     * #THROWS SailoException 
     * #import java.io.File;
     * 
     *  Pelaajat pelurit = new Pelaajat();
     *  Pelaaja aku1 = new Pelaaja(), aku2 = new Pelaaja();
     *  aku1.vastaaPasi();
     *  aku2.vastaaPasi();
     *  String hakemisto = "pelurit";
     *  String tiedNimi = hakemisto+"/pelaajat";
     *  File ftied = new File(tiedNimi+".dat");
     *  File dir = new File(hakemisto);
     *  dir.mkdir();
     *  ftied.delete();
     *  pelurit.lueTiedostosta(tiedNimi); #THROWS SailoException
     *  pelurit.lisaaPelaaja(aku1);
     *  pelurit.lisaaPelaaja(aku2);
     *  pelurit.tallenna();
     *  pelurit = new Pelaajat();            // Poistetaan vanhat luomalla uusi
     *  pelurit.lueTiedostosta(tiedNimi);  // johon ladataan tiedot tiedostosta.
     *  Iterator<Pelaaja> i = pelurit.iterator();
     *  i.next() === aku1;
     *  i.next() === aku2;
     *  i.hasNext() === false;
     *  pelurit.lisaaPelaaja(aku2);
     *  pelurit.tallenna();
     *  ftied.delete() === true;
     *  File fbak = new File(tiedNimi+".bak");
     *  fbak.delete() === true;
     *  dir.delete() === true;
     * </pre>
     */

    public void lueTiedostosta(String hakemisto) throws SailoException {
        setTiedostonPerusNimi(hakemisto);
        try ( BufferedReader br = new BufferedReader(new FileReader(getTiedostonNimi())) ) {
            kokoNimi = br.readLine();
            if ( kokoNimi == null ) throw new SailoException("Tiedostossa ei ole yhtään pelaajaa");
            String rivi = br.readLine();
            if ( rivi == null ) throw new SailoException("Maksimikoko puuttuu");
            // int maxKoko = Mjonot.erotaInt(rivi,10); // tehdään jotakin

            while ( (rivi = br.readLine()) != null ) {
                rivi = rivi.trim();
                if ( "".equals(rivi) || rivi.charAt(0) == ';' ) continue;
                Pelaaja pelaaja = new Pelaaja();
                pelaaja.parse(rivi); // voisi olla virhekäsittely
                lisaaPelaaja(pelaaja);
            }
            muutettu = false;
        } catch ( FileNotFoundException e ) {
            throw new SailoException("Tiedosto " + getTiedostonNimi() + " ei aukea");
        } catch ( IOException e ) {
            throw new SailoException("Ongelmia tiedoston kanssa: " + e.getMessage());
        }

    }

    
    /**
     * @throws SailoException poikkeuksen tullessa 
     */
    public void lueTiedostosta() throws SailoException{
        lueTiedostosta(getTiedostonPerusNimi());
    }
    
    /**
     * Tallentaa pelaajiston tiedostoon. (Kesken)
     * @throws SailoException jos talletus epäonnistuu
     */
    public void tallenna() throws SailoException {
        if ( !muutettu ) return;

        File fbak = new File(getBakNimi());
        File ftied = new File(getTiedostonNimi());
        fbak.delete(); // if .. System.err.println("Ei voi tuhota");
        ftied.renameTo(fbak); // if .. System.err.println("Ei voi nimetä");

        try ( PrintWriter fo = new PrintWriter(new FileWriter(ftied.getCanonicalPath())) ) {
            fo.println(getKokoNimi());
            fo.println(pelaajaLista.length);
            for (Pelaaja peluri: this) {
                fo.println(peluri.toString());
            }
            //} catch ( IOException e ) { // ei heitä poikkeusta
            //  throw new SailoException("Tallettamisessa ongelmia: " + e.getMessage());
        } catch ( FileNotFoundException ex ) {
            throw new SailoException("Tiedosto " + ftied.getName() + " ei aukea");
        } catch ( IOException ex ) {
            throw new SailoException("Tiedoston " + ftied.getName() + " kirjoittamisessa ongelmia");
        }

        muutettu = false;

    }
    
    /**
     * @author Sakari
     * @version 22.3.2025
     * @example
     * <pre name="test">
     * #THROWS SailoException 
     * #PACKAGEIMPORT
     * #import java.util.*;
     * 
     * Pelaajat pelurit = new Pelaajat();
     * Pelaaja aku1 = new Pelaaja(), aku2 = new Pelaaja();
     * aku1.rekisteroi(); aku2.rekisteroi();
     *
     * pelurit.lisaaPelaaja(aku1); 
     * pelurit.lisaaPelaaja(aku2); 
     * pelurit.lisaaPelaaja(aku1); 
     * 
     * StringBuffer ids = new StringBuffer(30);
     * for (Pelaaja peluri : pelurit)   // Kokeillaan for-silmukan toimintaa
     *   ids.append(" "+peluri.getId());           
     * 
     * String tulos = " " + aku1.getId() + " " + aku2.getId() + " " + aku1.getId();
     * 
     * ids.toString() === tulos; 
     * 
     * ids = new StringBuffer(30);
     * for (Iterator<Pelaaja>  i=pelurit.iterator(); i.hasNext(); ) { // ja iteraattorin toimintaa
     *   Pelaaja peluri = i.next();
     *   ids.append(" "+peluri.getId());           
     * }
     * 
     * ids.toString() === tulos;
     * 
     * Iterator<Pelaaja>  i=pelurit.iterator();
     * i.next() == aku1  === true;
     * i.next() == aku2  === true;
     * i.next() == aku1  === true;
     * 
     * i.next();  #THROWS NoSuchElementException
     *  
     * </pre>
     */

    public class PelaajatIterator implements Iterator<Pelaaja>{
        
        private int kohdalla = 0;
        
        
        /**
         * Onko olemassa vielä seuraavaa jäsentä
         * @return true jos on vielä jäseniä
         */
        @Override
        public boolean hasNext() {
            return kohdalla < getLkm();
        }
        
        /**
         * Annetaan seuraava jäsen
         * @return seuraava jäsen
         * @throws NoSuchElementException jos seuraavaa ei ole 
         */
        @Override
        public Pelaaja next() throws NoSuchElementException {
            if ( !hasNext() ) throw new NoSuchElementException("Ei oo");
            return annaPelaaja(kohdalla++);
        }
        
        /**
         * Tuhoamista ei ole toteutettu
         * @throws UnsupportedOperationException aina
         */
        @Override
        public void remove() throws UnsupportedOperationException {
            throw new UnsupportedOperationException("Ei poistella ;)");
        }



    }

    @Override
    public Iterator<Pelaaja> iterator() {
        return new PelaajatIterator();
    }
    
    /** 
     * Palauttaa hakuehtoa vastaavien pelaajien viitteet 
     * @param hakuehto hakuehto 
     * @param k etsittävän kentän indeksi  
     * @return tietorakenteen löytyneistä jäsenistä 
     * @example 
     * <pre name="test"> 
     * #THROWS SailoException  
     *   Pelaajat pelurit = new Pelaajat(); 
     *   Pelaaja pelaaja1 = new Pelaaja(); pelaaja1.parse("1|Ankka Aku"); 
     *   Pelaaja pelaaja2 = new Pelaaja(); pelaaja2.parse("2|Ankka Tupu"); 
     *   Pelaaja pelaaja3 = new Pelaaja(); pelaaja3.parse("3|Susi Sepe"); 
     *   Pelaaja pelaaja4 = new Pelaaja(); pelaaja4.parse("4|Ankka Iines");  
     *   pelurit.lisaaPelaaja(pelaaja1); pelurit.lisaaPelaaja(pelaaja2); pelurit.lisaaPelaaja(pelaaja3); 
     *   pelurit.lisaaPelaaja(pelaaja4); 
     *   // TODO: toistaiseksi palauttaa kaikki jäsenet 
     * </pre> 
     */ 
    @SuppressWarnings("unused")
    public Collection<Pelaaja> etsi(String hakuehto, int k) { 
        Collection<Pelaaja> loytyneet = new ArrayList<Pelaaja>(); 
        for (Pelaaja peluri : this) { 
            loytyneet.add(peluri);  
        } 
        return loytyneet; 
    }

    
    /**
     * Metodi, joka tulostaa kaikki pelaajalistassa olevat 
     * pelaajat
     */
    public void tulostaPelaajat() {
        System.out.println("Pelaajat:");
        for (Pelaaja peluri : pelaajaLista) {
            if (peluri != null) {
                System.out.println(peluri.getNimi());
            }
            
        }
    }
    
    /** Testipääohjelma Pelaajat-luokalle
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        //TODO
        Pelaaja pasi = new Pelaaja();
        pasi.vastaaPasi();
        Pelaaja pasi2 = new Pelaaja();
        pasi2.vastaaPasi();
        Pelaajat pelurit = new Pelaajat();
        pasi.rekisteroi();
        pasi2.rekisteroi();
        try {
            pelurit.lisaaPelaaja(pasi);
            pelurit.lisaaPelaaja(pasi2);
            pelurit.tulostaPelaajat();
        }
        catch (SailoException ex) {
            System.out.print(ex.getMessage());
        }
        pelurit.poistaPelaaja(pasi);
        pelurit.tulostaPelaajat();
        
    }

    
    
    
}
