package luokat;

import java.io.OutputStream;
import java.io.PrintStream;
import java.util.Random;

import fi.jyu.mit.ohj2.Mjonot;

/** Rata 
 * @author Sakari
 * @version 10.3.2025
 *
 */
public class Rata {
    private String nimi = "";
    private int id;
    private int parasTulos; 
    private int parhaanId;
    private int par;
    private static int seuraavaId = 1;
    
    
    
    /**
     * Radan muodostaja
     */
    public Rata() {
        //ei tarvita vielä
    }
    
    /**
     * @param parhaanId kuka heittänyt 
     * radalla parhaan tuloksen.
     * @example
     * <pre name="test">
     * Rata laajis = new Rata(1);
     * laajis.getParhaanId() === 1;
     * </pre>
     */
    public Rata(int parhaanId) {
        this.parhaanId = parhaanId;
    }
    
    /**
     * @return parhaan tuloksen heittäneen pelaajan id
     */
    public int getParhaanId() {
        return this.parhaanId;
    }
    
    /** Metodi, joka hakee radan id:n
     * @return radan id:n
     */
    public int getId() {
        return this.id;
    }
    
    /**
     * Asettaa tunnusnumeron, pitää huolen että seuraava
     * id tulee olemaan suurempi kuin tähän mennessä suurin 
     * @param nro miksi numeroksi asetetaan
     */
    public void setId(int nro) {
        this.id = nro;
        if (this.id >= seuraavaId) seuraavaId = this.id +1;
    }
    
    /**
     * Hakee radan parin 
     * @return radan parin 
     */
    public int getPar() {
        return this.par;
    }
    
    /**
     * @param par mikä asetetaan radan par-tulokseksi
     */
    public void setPar(int par) {
        this.par = par;
    }
    
      
    /**
     * @return radan parhaan tuloksen
     */
    public int getParasTulos() {
        return this.parasTulos;
    }
    /** Metodi, joka hakee pelaajan nimen
     * @return Pelaajan nimen
     * @example
     * <pre name="test">
     *   Rata laajis = new Rata();
     *   laajis.vastaaLaajis(1);
     *   laajis.getNimi() =R= "laajis .*";
     * </pre>
     */
    public String getNimi() {
        return this.nimi;
    }
    
    
    /** Antaa seuraavalle radalle oikean id:n
     * @return uuden radan id:n
     * @example
     * <pre name="test">
     * Rata makipeto = new Rata();
     * Rata makimahoton = new Rata();
     * Rata vyonsolki = new Rata();
     * makipeto.getId() === 0;
     * makipeto.rekisteroi();
     * makimahoton.rekisteroi();
     * vyonsolki.rekisteroi();
     * makipeto.getId() === 1;
     * makimahoton.getId() === 2;
     * vyonsolki.getId() === 3;
     * </pre>
     */
    public int rekisteroi() {
        this.id = seuraavaId;
        seuraavaId++;
        return id;
    }
    
    /*
     * Palauttaa radan tiedot merkkijonona,
     * jonka voi tallentaa tiedostoon.
     * @example
     * <pre name="test">
     *   Rata rata = new Rata();
     *   rata.parse("   2   |  Vesala  |   60  | 3 | 0 ");
     *   rata.toString()    === "2|Vesala|60|3|0";
     * </pre>

     */
    @Override
    public String toString() {
        return ("" + getId() + "|" + this.nimi + "|" + this.par + "|" + this.parhaanId + "|" + this.parasTulos);
    }
    
    /**
     * @param rivi miltä riviltä luetaan 
     *  @example
     * <pre name="test">
     *   Rata rata= new Rata();
     *   rata.parse("   2   |  Vesala  |   60  | 3 | 0 ");
     *   rata.getParhaanId() === 3;
     *   rata.toString()    === "2|Vesala|60|3|0";
     *   
     *   rata.rekisteroi();
     *   int n = rata.getId();
     *   rata.parse(""+(n+20));
     *   rata.rekisteroi();
     *   rata.getId() === n+20+1;
     *   rata.toString()     === "" + (n+20+1) + "|Vesala|60|3|0";
     * </pre>
     */

    public void parse(String rivi) {
        StringBuffer sb = new StringBuffer(rivi);
        setId(Mjonot.erota(sb, '|', getId()));
        this.nimi = Mjonot.erota(sb, '|', this.nimi);
        this.par = Mjonot.erota(sb, '|', this.par);
        this.parhaanId = Mjonot.erota(sb, '|', this.parhaanId);
        this.parasTulos = Mjonot.erota(sb, '|', this.parasTulos);
    }
    
    
    @Override
    public boolean equals(Object rata) {
        if (rata == null) return false; 
        return this.toString().equals(rata.toString());
    }
    
    @Override 
    public int hashCode() {
        return this.id;
    }
    
    
    
    /** Apumetodi, jolla sijoitetaan "laajis + arvottu nimi" radalle.
     * @param arvottuNimi radalle asetettava arvottu nimi
     */
    public void vastaaLaajis(String arvottuNimi) {
        this.nimi=arvottuNimi;
        this.parasTulos = arvottuNumero(); 
    }
    
    /** 
     * Ylläolevan apumetodin apumetodi, joka arpoo nimen.
     * @param i kenen parhaaksi tulokseksi laitetaan
     */
    public void vastaaLaajis(int i) {
        this.parhaanId = i;
        String uusiNimi = arvoNimi();
        vastaaLaajis(uusiNimi);
    }
    
    /** Apualiohjelma joka arpoo radalle 
     * 3-kirjaimisen nimen.
     * @return laajis + arvottu 3 kirjaiminen nimi.
     * @example
     * <pre name="test">
     * Rata laajis = new Rata();
     * laajis.arvoNimi() =R= "laajis .*";
     * </pre>
     */
    public String arvoNimi() {
        Random r = new Random();
        StringBuilder uusiNimi = new StringBuilder();
        uusiNimi.append("laajis ");
        String aakkoset = "abcdefghijklmnopqrstuvxyz";
        for (int i = 0; i < 3; i++) {
            uusiNimi.append(aakkoset.charAt(r.nextInt(aakkoset.length())));
        }
        return uusiNimi.toString();
    }
    
    /**
     * @return arvotun numeron
     */
    public int arvottuNumero() {
        int palautus = random(-10, 10);
        return palautus;
    }
    
    /**
     * @param ala alaraja
     * @param yla ylaraja
     * @return random kokonaisluvun valitulta väliltä
     */
    public int random(int ala, int yla) {
        double rand = (yla - ala)*Math.random() +ala;
        return (int)Math.round(rand);
    }
    
    /**
     * Tulostetaan radan tiedot
     * @param out tietovirta johon tulostetaan
     */
    public void tulosta(PrintStream out) {
        out.println(String.format("%03d", this.id) + "  " + this.nimi); 
    }
    
    /**
     * Tulostetaan radan tiedot
     * @param os tietovirta johon tulostetaan
     */
    public void tulosta(OutputStream os) {
        tulosta(new PrintStream(os));
    }
    

    
    
    /** Testipääohjelma Rata-luokalle
     * @param args ei käytössä
     */
    public static void main(String[] args) {
//        Rata laajis = new Rata();
//        laajis.vastaaLaajis(1);
//        laajis.tulosta(System.out);
//        laajis.rekisteroi();
//        int p = laajis.getParhaanId();
//        System.out.println(p);
//        laajis.tulosta(System.out);
//        Rata e = new Rata();
//        e.tulosta(System.out);
//        e.rekisteroi();
//        e.tulosta(System.out);
        Rata makipeto = new Rata();
        Rata makimahoton = new Rata();
        Rata vyonsolki = new Rata();
        System.out.println(makipeto.getId());
        makipeto.rekisteroi();
        makimahoton.rekisteroi();
        vyonsolki.rekisteroi();
        System.out.println(makipeto.getId());
        System.out.println(makimahoton.getId());
        System.out.println(vyonsolki.getId());
    }

}
