package luokat;

import java.util.List;

/** Tulskortti-luokka. Huolehtii pelaajistosta,
 * metodit välittäjämetodeja pelaajistoon. 
 * @author Sakari
 * @version 5.3.2025
 *
 */
public class Tuloskortti {

    private final Pelaajat pelaajat = new Pelaajat();
    private final Radat radat = new Radat();
    
    
    /**
     * @return pelaajien lukumäärän 
     */
    public int getPelaajia() {
        return pelaajat.getLkm();
    }
    
    /**
     * Palauttaa i:n jäsenen
     * @param i monesko jäsen palautetaan
     * @return viite i:teen jäseneen
     * @throws IndexOutOfBoundsException jos i väärin
     */
    public Pelaaja annaPelaaja(int i) throws IndexOutOfBoundsException {
        return pelaajat.annaPelaaja(i);
    }

    
    
    /**
     * Lukee pelaajat tiedostosta. (Kesken)
     * @param hakemisto missä pelaajat.dat sijaitsee
     * @throws SailoException jos lukeminen epäonnistuu
     */
    public void lueTiedostosta(String hakemisto) throws SailoException {
        pelaajat.lueTiedostosta(hakemisto);
    }
    
    
    /** Tallentaa tuloskortin tiedot tiedostoon 
     * @throws SailoException jos tallentaminen ei onnistu 
     */
    public void tallenna() throws SailoException {
        pelaajat.tallenna();
    }
    
    /**
     * Tulostaa kaikki pelaajat, vaikka nimi olisi null
     */
    public void tulostaPelaajat() {
        pelaajat.tulostaPelaajat();
    }
    
    
    
    /**
     * @param pelaaja kenen ennätysradat halutaan 
     * @return radat, joissa pelaajalla on ennätys
     * @example
     * <pre name="test">
     * #import java.util.*;
     * 
     * Tuloskortti tuloskortti = new Tuloskortti();
     * Pelaaja pasi1 = new Pelaaja();
     * Pelaaja pasi2 = new Pelaaja();
     * Pelaaja pasi3 = new Pelaaja();
     * pasi1.rekisteroi(); pasi2.rekisteroi(); pasi3.rekisteroi();
     * int id1 = pasi1.getId();
     * int id2 = pasi2.getId(); 
     * Rata laajis01 = new Rata(id1); tuloskortti.lisaaRata(laajis01);
     * Rata laajis02 = new Rata(id1); tuloskortti.lisaaRata(laajis02);
     * Rata laajis03 = new Rata(id2); tuloskortti.lisaaRata(laajis03);
     * Rata laajis04 = new Rata(id2); tuloskortti.lisaaRata(laajis04);
     * Rata laajis05 = new Rata(id2); tuloskortti.lisaaRata(laajis05);
     * 
     * List<Rata> loytyneet;
     * loytyneet = tuloskortti.annaRadat(pasi3);
     * loytyneet.size() === 0;
     * loytyneet = tuloskortti.annaRadat(pasi1);
     * loytyneet.size() === 2;
     * loytyneet.get(0) == laajis01 === true;
     * loytyneet.get(1) == laajis02 === true;
     * loytyneet = tuloskortti.annaRadat(pasi2);
     * loytyneet.size() === 3;
     * loytyneet.get(0) == laajis03 === true;
     * loytyneet.get(1) == laajis04 === true;
     * loytyneet.get(2) == laajis05 === true;
     * </pre>
     */
    public List<Rata> annaRadat(Pelaaja pelaaja){
        return radat.annaRadat(pelaaja.getId());
    }
    
    /**
     * Lisää tuloskorttiin uuden pelaajan 
     * @param pelaaja lisättävä pelaaja
     * @throws SailoException jos pelaajalista on jo täynnä
     * @example
     * <pre name="test">
     * #THROWS SailoException 
     * Pelaajat pelurit = new Pelaajat();
     * Pelaaja pasi1 = new Pelaaja(), pasi2 = new Pelaaja();
     * pelurit.getLkm() === 0;
     * pelurit.lisaaPelaaja(pasi1); pelurit.getLkm() === 1;
     * pelurit.lisaaPelaaja(pasi2); pelurit.getLkm() === 2;
     * pelurit.lisaaPelaaja(pasi1); pelurit.getLkm() === 3;
     * pelurit.annaPelaaja(0) === pasi1;
     * pelurit.annaPelaaja(1) === pasi2;
     * pelurit.annaPelaaja(2) === pasi1;
     * pelurit.annaPelaaja(1) == pasi1 === false;
     * pelurit.annaPelaaja(1) == pasi2 === true;
     * pelurit.annaPelaaja(3) === pasi1; #THROWS IndexOutOfBoundsException 
     * pelurit.lisaaPelaaja(pasi1); pelurit.getLkm() === 4;
     * pelurit.lisaaPelaaja(pasi1);  #THROWS SailoException
     * </pre>
     */
    public void lisaaPelaaja(Pelaaja pelaaja) throws SailoException {
        pelaajat.lisaaPelaaja(pelaaja);
    }
    
    /**
     * @param rata r
     * @example
     * <pre name="test">
     * #import java.util.*;
     * 
     * Radat radat = new Radat();
     * Rata laajis1 = new Rata(1); 
     * Rata laajis2 = new Rata(2);
     * radat.koko() === 0;
     * radat.lisaaRata(laajis1); radat.koko() === 1;
     * radat.lisaaRata(laajis2); radat.koko() === 2;
     * 
     * List<Rata> loytyneet;
     * loytyneet = radat.annaRadat(5);
     * loytyneet.size() === 0;
     * loytyneet = radat.annaRadat(1);
     * loytyneet.size() === 1;
     * loytyneet.get(0) == laajis1 === true;
     * </pre>
     */
    public void lisaaRata(Rata rata) {
        radat.lisaaRata(rata);
    }
    
    
    
    
    
    
    /** Poistaa pelaajan tuloskortista, 
     * mikäli pelaaja ylipäätään on pelaajalistassa.
     * @param pelaaja pelaaja joka yritetään poistaa
     * @example
     * <pre name="test">
     * #THROWS SailoException
     * Pelaajat pelurit = new Pelaajat();
     * Pelaaja pasi1 = new Pelaaja(), pasi2 = new Pelaaja(), pasi3 = new Pelaaja();
     * pasi1.rekisteroi();
     * pasi2.rekisteroi();
     * pasi3.rekisteroi();
     * pelurit.lisaaPelaaja(pasi1);
     * pelurit.lisaaPelaaja(pasi2);
     * pelurit.lisaaPelaaja(pasi3);
     * pelurit.getLkm() === 3;
     * pelurit.poistaPelaaja(pasi2);
     * pelurit.getLkm() === 2;
     * pelurit.poistaPelaaja(pasi3);
     * pelurit.getLkm() === 1;
     * pelurit.poistaPelaaja(pasi1);
     * pelurit.getLkm() === 0;
     * </pre>
     */
    public void poistaPelaaja(Pelaaja pelaaja) {
        pelaajat.poistaPelaaja(pelaaja);
    }
    
    /** Tuloskortin testiohjelma 
     * @param args ei käytöösä
     */
    public static void main(String args[]) {
        Tuloskortti tuloskortti = new Tuloskortti();
        
        try {
            Pelaaja pasi = new Pelaaja();
            pasi.vastaaPasi();
            pasi.rekisteroi();
            Pelaaja pasi2 = new Pelaaja();
            pasi2.vastaaPasi();
            pasi2.rekisteroi();
            
            tuloskortti.lisaaPelaaja(pasi);
            tuloskortti.lisaaPelaaja(pasi2);
            tuloskortti.tulostaPelaajat();
            
            Rata laajis = new Rata();
            laajis.vastaaLaajis(pasi.getId());
            Rata keljo = new Rata();
            keljo.vastaaLaajis(pasi.getId());
            Rata seppala = new Rata();
            seppala.vastaaLaajis(pasi2.getId());
            
            tuloskortti.lisaaRata(laajis);
            laajis.rekisteroi();
            tuloskortti.lisaaRata(keljo);
            keljo.rekisteroi();
            tuloskortti.lisaaRata(seppala);
            seppala.rekisteroi();
            
            
            for (int i = 0; i < tuloskortti.getPelaajia(); i++) {
                Pelaaja pelaaja= tuloskortti.annaPelaaja(i);
                System.out.println("Jäsen paikassa: " + i);
                pelaaja.tulosta(System.out);
                
                List <Rata> radat2 = tuloskortti.annaRadat(pelaaja);
                
                for (Rata rata : radat2) {
                    System.out.print(rata.getParhaanId() + " ");
                    rata.tulosta(System.out);
                }
                
            }

        
        } catch (SailoException ex) {
            System.out.print(ex.getMessage());
        }
        
    }
    
}
